@@ -13,10 +13,14 @@ import collections
 import os
 import types
 import zipimport
+from collections.abc import Callable, Iterator, Sequence
 from importlib.util import find_spec, module_from_spec
-from typing import TYPE_CHECKING, ClassVar
+from typing import Any, ClassVar
 
+from astroid import nodes
+from astroid._cache import CACHE_MANAGER
 from astroid.const import BRAIN_MODULES_DIRECTORY
+from astroid.context import InferenceContext, _invalidate_cache
 from astroid.exceptions import AstroidBuildingError, AstroidImportError
 from astroid.interpreter._import import spec, util
 from astroid.modutils import (
@@ -26,20 +30,17 @@ from astroid.modutils import (
     get_source_file,
     is_module_name_part_of_extension_package_whitelist,
     is_python_source,
-    is_standard_module,
+    is_stdlib_module,
     load_module_from_name,
     modpath_from_file,
 )
 from astroid.transforms import TransformVisitor
-from astroid.typing import AstroidManagerBrain
-
-if TYPE_CHECKING:
-    from astroid import nodes
+from astroid.typing import AstroidManagerBrain, InferenceResult
 
 ZIP_IMPORT_EXTS = (".zip", ".egg", ".whl", ".pyz", ".pyzw")
 
 
-def safe_repr(obj):
+def safe_repr(obj: Any) -> str:
     try:
         return repr(obj)
     except Exception:  # pylint: disable=broad-except
@@ -89,12 +90,18 @@ class AstroidManager:
     def builtins_module(self) -> nodes.Module:
         return self.astroid_cache["builtins"]
 
-    def visit_transforms(self, node):
+    def visit_transforms(self, node: nodes.NodeNG) -> InferenceResult:
         """Visit the transforms and apply them to the given *node*."""
         return self._transform.visit(node)
 
-    def ast_from_file(self, filepath, modname=None, fallback=True, source=False):
-        """given a module name, return the astroid object"""
+    def ast_from_file(
+        self,
+        filepath: str,
+        modname: str | None = None,
+        fallback: bool = True,
+        source: bool = False,
+    ) -> nodes.Module:
+        """Given a module name, return the astroid object."""
         try:
             filepath = get_source_file(filepath, include_no_ext=True)
             source = True
@@ -119,20 +126,26 @@ class AstroidManager:
             return self.ast_from_module_name(modname)
         raise AstroidBuildingError("Unable to build an AST for {path}.", path=filepath)
 
-    def ast_from_string(self, data, modname="", filepath=None):
-        """Given some source code as a string, return its corresponding astroid object"""
+    def ast_from_string(
+        self, data: str, modname: str = "", filepath: str | None = None
+    ) -> nodes.Module:
+        """Given some source code as a string, return its corresponding astroid
+        object.
+        """
         # pylint: disable=import-outside-toplevel; circular import
         from astroid.builder import AstroidBuilder
 
         return AstroidBuilder(self).string_build(data, modname, filepath)
 
-    def _build_stub_module(self, modname):
+    def _build_stub_module(self, modname: str) -> nodes.Module:
         # pylint: disable=import-outside-toplevel; circular import
         from astroid.builder import AstroidBuilder
 
         return AstroidBuilder(self).string_build("", modname)
 
-    def _build_namespace_module(self, modname: str, path: list[str]) -> nodes.Module:
+    def _build_namespace_module(
+        self, modname: str, path: Sequence[str]
+    ) -> nodes.Module:
         # pylint: disable=import-outside-toplevel; circular import
         from astroid.builder import build_namespace_package_module
 
@@ -141,15 +154,26 @@ class AstroidManager:
     def _can_load_extension(self, modname: str) -> bool:
         if self.always_load_extensions:
             return True
-        if is_standard_module(modname):
+        if is_stdlib_module(modname):
             return True
         return is_module_name_part_of_extension_package_whitelist(
             modname, self.extension_package_whitelist
         )
 
-    def ast_from_module_name(self, modname, context_file=None):
-        """given a module name, return the astroid object"""
-        if modname in self.astroid_cache:
+    def ast_from_module_name(  # noqa: C901
+        self,
+        modname: str | None,
+        context_file: str | None = None,
+        use_cache: bool = True,
+    ) -> nodes.Module:
+        """Given a module name, return the astroid object."""
+        if modname is None:
+            raise AstroidBuildingError("No module name given.")
+        # Sometimes we don't want to use the cache. For example, when we're
+        # importing a module with the same name as the file that is importing
+        # we want to fallback on the import system to make sure we get the correct
+        # module.
+        if modname in self.astroid_cache and use_cache:
             return self.astroid_cache[modname]
         if modname == "__main__":
             return self._build_stub_module(modname)
@@ -173,14 +197,14 @@ class AstroidManager:
                 ):
                     return self._build_stub_module(modname)
                 try:
-                    module = load_module_from_name(modname)
+                    named_module = load_module_from_name(modname)
                 except Exception as e:
                     raise AstroidImportError(
                         "Loading {modname} failed with:\n{error}",
                         modname=modname,
                         path=found_spec.location,
                     ) from e
-                return self.ast_from_module(module, modname)
+                return self.ast_from_module(named_module, modname)
 
             elif found_spec.type == spec.ModuleType.PY_COMPILED:
                 raise AstroidImportError(
@@ -191,7 +215,7 @@ class AstroidManager:
 
             elif found_spec.type == spec.ModuleType.PY_NAMESPACE:
                 return self._build_namespace_module(
-                    modname, found_spec.submodule_search_locations
+                    modname, found_spec.submodule_search_locations or []
                 )
             elif found_spec.type == spec.ModuleType.PY_FROZEN:
                 if found_spec.location is None:
@@ -217,7 +241,7 @@ class AstroidManager:
             if context_file:
                 os.chdir(old_cwd)
 
-    def zip_import_data(self, filepath):
+    def zip_import_data(self, filepath: str) -> nodes.Module | None:
         if zipimport is None:
             return None
 
@@ -244,7 +268,9 @@ class AstroidManager:
                 continue
         return None
 
-    def file_from_module_name(self, modname, contextfile):
+    def file_from_module_name(
+        self, modname: str, contextfile: str | None
+    ) -> spec.ModuleSpec:
         try:
             value = self._mod_file_cache[(modname, contextfile)]
         except KeyError:
@@ -266,8 +292,10 @@ class AstroidManager:
             raise value.with_traceback(None)  # pylint: disable=no-member
         return value
 
-    def ast_from_module(self, module: types.ModuleType, modname: str | None = None):
-        """given an imported module, return the astroid object"""
+    def ast_from_module(
+        self, module: types.ModuleType, modname: str | None = None
+    ) -> nodes.Module:
+        """Given an imported module, return the astroid object."""
         modname = modname or module.__name__
         if modname in self.astroid_cache:
             return self.astroid_cache[modname]
@@ -275,7 +303,8 @@ class AstroidManager:
             # some builtin modules don't have __file__ attribute
             filepath = module.__file__
             if is_python_source(filepath):
-                return self.ast_from_file(filepath, modname)
+                # Type is checked in is_python_source
+                return self.ast_from_file(filepath, modname)  # type: ignore[arg-type]
         except AttributeError:
             pass
 
@@ -284,8 +313,8 @@ class AstroidManager:
 
         return AstroidBuilder(self).module_build(module, modname)
 
-    def ast_from_class(self, klass, modname=None):
-        """get astroid for the given class"""
+    def ast_from_class(self, klass: type, modname: str | None = None) -> nodes.ClassDef:
+        """Get astroid for the given class."""
         if modname is None:
             try:
                 modname = klass.__module__
@@ -297,14 +326,24 @@ class AstroidManager:
                     modname=modname,
                 ) from exc
         modastroid = self.ast_from_module_name(modname)
-        return modastroid.getattr(klass.__name__)[0]  # XXX
+        ret = modastroid.getattr(klass.__name__)[0]
+        assert isinstance(ret, nodes.ClassDef)
+        return ret
 
-    def infer_ast_from_something(self, obj, context=None):
-        """infer astroid for the given class"""
+    def infer_ast_from_something(
+        self, obj: object, context: InferenceContext | None = None
+    ) -> Iterator[InferenceResult]:
+        """Infer astroid for the given class."""
         if hasattr(obj, "__class__") and not isinstance(obj, type):
             klass = obj.__class__
-        else:
+        elif isinstance(obj, type):
             klass = obj
+        else:
+            raise AstroidBuildingError(  # pragma: no cover
+                "Unable to get type for {class_repr}.",
+                cls=None,
+                class_repr=safe_repr(obj),
+            )
         try:
             modname = klass.__module__
         except AttributeError as exc:
@@ -343,7 +382,7 @@ class AstroidManager:
             for inferred in modastroid.igetattr(name, context):
                 yield inferred.instantiate_class()
 
-    def register_failed_import_hook(self, hook):
+    def register_failed_import_hook(self, hook: Callable[[str], nodes.Module]) -> None:
         """Registers a hook to resolve imports that cannot be found otherwise.
 
         `hook` must be a function that accepts a single argument `modname` which
@@ -353,12 +392,12 @@ class AstroidManager:
         """
         self._failed_import_hooks.append(hook)
 
-    def cache_module(self, module):
+    def cache_module(self, module: nodes.Module) -> None:
         """Cache a module if no module with the same name is known yet."""
         self.astroid_cache.setdefault(module.name, module)
 
     def bootstrap(self) -> None:
-        """Bootstrap the required AST modules needed for the manager to work
+        """Bootstrap the required AST modules needed for the manager to work.
 
         The bootstrap usually involves building the AST for the builtins
         module, which is required by the rest of astroid to work correctly.
@@ -368,25 +407,31 @@ class AstroidManager:
         raw_building._astroid_bootstrapping()
 
     def clear_cache(self) -> None:
-        """Clear the underlying cache, bootstrap the builtins module and
-        re-register transforms."""
+        """Clear the underlying caches, bootstrap the builtins module and
+        re-register transforms.
+        """
         # import here because of cyclic imports
         # pylint: disable=import-outside-toplevel
         from astroid.inference_tip import clear_inference_tip_cache
         from astroid.interpreter.objectmodel import ObjectModel
         from astroid.nodes.node_classes import LookupMixIn
+        from astroid.nodes.scoped_nodes import ClassDef
 
         clear_inference_tip_cache()
+        _invalidate_cache()  # inference context cache
 
         self.astroid_cache.clear()
         # NB: not a new TransformVisitor()
         AstroidManager.brain["_transform"].transforms = collections.defaultdict(list)
 
+        CACHE_MANAGER.clear_all_caches()
+
         for lru_cache in (
             LookupMixIn.lookup,
             _cache_normalize_path_,
             util.is_namespace,
             ObjectModel.attributes,
+            ClassDef._metaclass_lookup_attribute,
         ):
             lru_cache.cache_clear()  # type: ignore[attr-defined]
 
